﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace uEye_DotNet_GPIO_MasterSlave
{
    public partial class uEye_DotNet_GPIO_EngineeringSample : Form
    {
        uEye.Defines.Status statusRet = 0; 
        private uEye.Camera CameraMaster, CameraSlave;
        IntPtr displayHandleMaster = IntPtr.Zero;
        IntPtr displayHandleSlave = IntPtr.Zero;
               
        // PWM params
        double f64Frequency, f64DutyCycle;

        public uEye_DotNet_GPIO_EngineeringSample()
        {
            InitializeComponent();
            displayHandleMaster = pictureBoxMaster.Handle;
            displayHandleSlave = pictureBoxSlave.Handle;

            InitCamera();
        }

        /// <summary>
        /// Initializes the Master Camera (ID=1) and the Slave Camera (ID=2), incl. memory allocation
        /// </summary>
        private void InitCamera()
        {
            CameraMaster = new uEye.Camera();
            CameraSlave = new uEye.Camera();

            f64DutyCycle = 0.5;

            // Open Master Camera
            statusRet = CameraMaster.Init(1, displayHandleMaster);
            if (statusRet != uEye.Defines.Status.Success)
            {
                MessageBox.Show("Initialization of Master camera (ID = 1) failed");
                Environment.Exit(-1);
            }

            // Reset PWM modes
            statusRet = CameraMaster.IO.Pwm.SetMode(0);

            # region GUI: PWM sliders
            // Get PWM Values for GUI
            double fMin, fMax, fInc;
            statusRet = CameraMaster.IO.Pwm.GetFrequencyRange(out fMin, out fMax, out fInc);
            double dcMin, dcMax, dcInc;
            statusRet = CameraMaster.IO.Pwm.GetDutyCycleRange(out dcMin, out dcMax, out dcInc);

            trackBar_PWMFrequency.SetRange(Convert.ToInt32(fMin), Convert.ToInt32(fMax));
            trackBar_PWMFrequency.TickFrequency = Convert.ToInt32(fInc);
            label_fMin.Text = fMin.ToString() + " Hz";
            label_fMax.Text = fMax.ToString() + " Hz";
            numericUpDown_PWMFrequency.Minimum = Convert.ToInt32(fMin);
            numericUpDown_PWMFrequency.Maximum = Convert.ToInt32(fMax);

            trackBar_DutyCycle.SetRange(Convert.ToInt32(dcMin*100), Convert.ToInt32(dcMax*100));
            trackBar_DutyCycle.TickFrequency = Convert.ToInt32(dcInc);
            trackBar_DutyCycle.Value = 50;

            label_DutyCycleMin.Text = (dcMin*100).ToString() + " %";
            label_DutyCycleMax.Text = (dcMax*100).ToString() + " %";
            label_PWM_DutyCycle.Text = "0.5";
            # endregion GUI: PWM sliders

            // Allocate Memory
            statusRet = CameraMaster.Memory.Allocate();
            if (statusRet != uEye.Defines.Status.Success)
            {
                MessageBox.Show("Allocate Memory failed");
                Environment.Exit(-1);
            }

            // Start Live Video
            statusRet = CameraMaster.Acquisition.Capture();
            if (statusRet != uEye.Defines.Status.Success)
            {
                MessageBox.Show("Start Live Video failed");
            }

            // Init Slave camera
            statusRet = CameraSlave.Init(2, displayHandleSlave);
            statusRet = CameraSlave.Memory.Allocate();

            // Connect Events
            CameraSlave.EventFrame += onFrameEventSlave;
            CameraMaster.EventFrame += onFrameEventMaster;
        }

        # region Frame Events
        private void onFrameEventMaster(object sender, EventArgs e)
        {
            uEye.Camera CameraMaster = sender as uEye.Camera;
            uEye.Types.ImageInfo ImageInfoMaster; 

            Int32 s32MemIDMaster;
            CameraMaster.Memory.GetActive(out s32MemIDMaster);
            CameraMaster.Information.GetImageInfo(s32MemIDMaster, out ImageInfoMaster);
           
            textBox_TimestampMaster.BeginInvoke(new Action(delegate() { textBox_TimestampMaster.Text = ImageInfoMaster.TimestampSystem.ToString() + ":" + ImageInfoMaster.TimestampSystem.Millisecond; ; }));
            CameraMaster.Display.Render(s32MemIDMaster, displayHandleMaster, uEye.Defines.DisplayRenderMode.FitToWindow);
        }


        private void onFrameEventSlave(object sender, EventArgs e)
        {
            uEye.Camera CameraSlave = sender as uEye.Camera;
            uEye.Types.ImageInfo ImageInfoSlave; 

            Int32 s32MemIDSlave;
            CameraSlave.Memory.GetActive(out s32MemIDSlave);
            CameraSlave.Information.GetImageInfo(s32MemIDSlave, out ImageInfoSlave);

            textBox_TimestampSlave.BeginInvoke(new Action(delegate() { textBox_TimestampSlave.Text = ImageInfoSlave.TimestampSystem.ToString() + ":" + ImageInfoSlave.TimestampSystem.Millisecond; ; }));
            CameraSlave.Display.Render(s32MemIDSlave, displayHandleSlave, uEye.Defines.DisplayRenderMode.FitToWindow);
        }
        # endregion Frame Events


        // btn: Exit
        private void btn_Exit_Click(object sender, EventArgs e)
        {
            CameraSlave.EventFrame -= onFrameEventSlave;
            CameraMaster.EventFrame -= onFrameEventMaster;
            CameraMaster.Exit();
            CameraSlave.Exit();
            Close();
        }


        // btn: Configure Master Camera - ID1
        private void btn_ConfigureMaster_Click(object sender, EventArgs e)
        {
            // Set Trigger mode rising edge
            statusRet = CameraMaster.Trigger.Set(uEye.Defines.TriggerMode.Lo_Hi);

            // GPIO2 as PWM output
            SetPWMOutput();

            // GPIO1 as trigger input 
            statusRet = CameraMaster.IO.Gpio.SetConfiguration(uEye.Defines.IO.GPIO.One, uEye.Defines.IO.GPIOConfiguration.Trigger);
            statusRet = CameraMaster.Acquisition.Capture();        
        }

        private void SetPWMOutput()
        {
            statusRet = CameraMaster.IO.Pwm.SetMode(uEye.Defines.IO.PwmMode.Gpio2);
            statusRet = CameraMaster.IO.Pwm.SetParams(f64Frequency, f64DutyCycle);
        }

            private void trackBar_PWMFrequency_Scroll(object sender, EventArgs e)
        {
            label_PWMFrequency.Text = trackBar_PWMFrequency.Value.ToString();
            numericUpDown_PWMFrequency.Value = trackBar_PWMFrequency.Value;
            f64Frequency = trackBar_PWMFrequency.Value;
            SetPWMOutput();
        }

            private void numericUpDown_PWMFrequency_ValueChanged(object sender, EventArgs e)
        {
            f64Frequency = Convert.ToDouble(numericUpDown_PWMFrequency.Value);
            trackBar_PWMFrequency.Value = Convert.ToInt32(numericUpDown_PWMFrequency.Value);
            label_PWMFrequency.Text = numericUpDown_PWMFrequency.Value.ToString();
            SetPWMOutput();
        }

        private void trackBar_DutyCycle_Scroll(object sender, EventArgs e)
        {
            f64DutyCycle = (trackBar_DutyCycle.Value)/100.0;
            label_PWM_DutyCycle.Text = f64DutyCycle.ToString();
            SetPWMOutput();
        }

        // btn: Configure Slave Camera - ID2
        private void btn_ConfigureSlave_Click(object sender, EventArgs e)
        {
            statusRet = CameraSlave.Acquisition.Stop(uEye.Defines.DeviceParameter.Wait);

            // Set Trigger mode rising edge
            statusRet = CameraSlave.Trigger.Set(uEye.Defines.TriggerMode.Lo_Hi);

            // GPIO1 as trigger input 
            statusRet = CameraSlave.IO.Gpio.SetConfiguration(uEye.Defines.IO.GPIO.One, uEye.Defines.IO.GPIOConfiguration.Trigger);

            statusRet = CameraSlave.Acquisition.Capture();  
        }

    }
}
